@@ -5,8 +5,9 @@
 #include <asm/elf.h>
 #include <asm/types.h>
 #include "asm/types.h"
+#include <compel/asm/infect-types.h>
 
-#include "sigframe.h"
+#include <compel/asm/sigframe.h>
 
 /*
  * Clone trampoline
@@ -47,6 +48,7 @@
 		  "r"(&thread_args[i])		/* %6 */		\
 		: "memory","0","3","4","5","6","7","14","15")
 
+#define kdat_compat_sigreturn_test()			0
 
 int restore_gpregs(struct rt_sigframe *f, UserPpc64RegsEntry *r);
 int restore_nonsigframe_gpregs(UserPpc64RegsEntry *r);
@@ -54,19 +56,14 @@ int restore_nonsigframe_gpregs(UserPpc64RegsEntry *r);
 /* Nothing to do, TLS is accessed through r13 */
 static inline void restore_tls(tls_t *ptls) { (void)ptls; }
 
-static inline int ptrace_set_breakpoint(pid_t pid, void *addr)
-{
-        return 0;
-}
-
-static inline int ptrace_flush_breakpoints(pid_t pid)
-{
-        return 0;
-}
-
 /*
  * Defined in arch/ppc64/syscall-common-ppc64.S
  */
 unsigned long sys_shmat(int shmid, const void *shmaddr, int shmflg);
 
+static inline void *alloc_compat_syscall_stack(void) { return NULL; }
+static inline void free_compat_syscall_stack(void *stack32) { }
+static inline int
+arch_compat_rt_sigaction(void *stack, int sig, void *act) { return -1; }
+
 #endif /*__CR_ASM_RESTORER_H__*/
