@@ -2,12 +2,22 @@
 #define __CR_ASM_RESTORER_H__
 
 #include "asm/types.h"
-#include "asm/fpu.h"
+#include <compel/asm/fpu.h>
 #include "images/core.pb-c.h"
+#include <compel/plugins/std/syscall-codes.h>
+#include <compel/asm/sigframe.h>
+#include "asm/compat.h"
 
-#include "sigframe.h"
+#ifdef CONFIG_COMPAT
+extern void restore_tls(tls_t *ptls);
+extern int arch_compat_rt_sigaction(void *stack32, int sig,
+		rt_sigaction_t_compat *act);
+#else /* CONFIG_COMPAT */
+static inline void restore_tls(tls_t *ptls) { }
+static inline int
+arch_compat_rt_sigaction(void *stack, int sig, void *act) { return -1; }
+#endif /* !CONFIG_COMPAT */
 
-#ifdef CONFIG_X86_64
 #define RUN_CLONE_RESTORE_FN(ret, clone_flags, new_sp, parent_tid,      \
 			     thread_args, clone_restore_fn)             \
 	asm volatile(							\
@@ -54,31 +64,45 @@
 		     :						\
 		     : "r"(ret)					\
 		     : "memory")
-#else /* CONFIG_X86_64 */
-#define RUN_CLONE_RESTORE_FN(ret, clone_flags, new_sp, parent_tid,      \
-			     thread_args, clone_restore_fn)             \
-	(void)ret;							\
-	(void)clone_flags;						\
-	(void)new_sp;							\
-	(void)parent_tid;						\
-	(void)thread_args;						\
-	(void)clone_restore_fn;						\
-	;
-#define ARCH_FAIL_CORE_RESTORE					\
-	asm volatile(						\
-		     "movl %0, %%esp			    \n"	\
-		     "xorl %%eax, %%eax			    \n"	\
-		     "jmp *%%eax			    \n"	\
-		     :						\
-		     : "r"(ret)					\
-		     : "memory")
-#endif /* CONFIG_X86_64 */
+
+#ifndef ARCH_MAP_VDSO_32
+# define ARCH_MAP_VDSO_32		0x2002
+#endif
+
+extern int kdat_compat_sigreturn_test(void);
+
+static inline void
+__setup_sas_compat(struct ucontext_ia32* uc, ThreadSasEntry *sas)
+{
+	uc->uc_stack.ss_sp	= (compat_uptr_t)(sas)->ss_sp;
+	uc->uc_stack.ss_flags	= (int)(sas)->ss_flags;
+	uc->uc_stack.ss_size	= (compat_size_t)(sas)->ss_size;
+}
+
+static inline void
+__setup_sas(struct rt_sigframe* sigframe, ThreadSasEntry *sas)
+{
+	if (sigframe->is_native) {
+		struct rt_ucontext *uc	= &sigframe->native.uc;
+
+		uc->uc_stack.ss_sp	= (void *)decode_pointer((sas)->ss_sp);
+		uc->uc_stack.ss_flags	= (int)(sas)->ss_flags;
+		uc->uc_stack.ss_size	= (size_t)(sas)->ss_size;
+	} else {
+		__setup_sas_compat(&sigframe->compat.uc, sas);
+	}
+}
+
+static inline void _setup_sas(struct rt_sigframe* sigframe, ThreadSasEntry *sas)
+{
+	if (sas)
+		__setup_sas(sigframe, sas);
+}
+#define setup_sas _setup_sas
 
 int restore_gpregs(struct rt_sigframe *f, UserX86RegsEntry *r);
 int restore_nonsigframe_gpregs(UserX86RegsEntry *r);
 
-static inline void restore_tls(tls_t *ptls) { (void)ptls; }
-
 int ptrace_set_breakpoint(pid_t pid, void *addr);
 int ptrace_flush_breakpoints(pid_t pid);
 
