@@ -2,8 +2,8 @@
 #define __CR_PROC_PARSE_H__
 
 #include <sys/types.h>
-#include "asm/types.h"
 
+#include <compel/compel.h>
 #include "images/seccomp.pb-c.h"
 
 #define PROC_TASK_COMM_LEN	32
@@ -73,15 +73,11 @@ struct seccomp_info {
 #define PROC_CAP_SIZE	2
 
 struct proc_status_creds {
+	struct seize_task_status s;
+
 	unsigned int uids[4];
 	unsigned int gids[4];
 
-	char			state;
-	int			ppid;
-	unsigned long long	sigpnd;
-	unsigned long long	shdpnd;
-
-	int			seccomp_mode;
 	u32			last_filter;
 
 	/*
@@ -100,7 +96,7 @@ extern int parse_pid_stat(pid_t pid, struct proc_pid_stat *s);
 extern unsigned int parse_pid_loginuid(pid_t pid, int *err, bool ignore_noent);
 extern int parse_pid_oom_score_adj(pid_t pid, int *err);
 extern int prepare_loginuid(unsigned int value, unsigned int loglevel);
-extern int parse_pid_status(pid_t pid, struct proc_status_creds *);
+extern int parse_pid_status(pid_t pid, struct seize_task_status *);
 extern int parse_file_locks(void);
 extern int get_fd_mntid(int fd, int *mnt_id);
 
